/*************************************************
 *	 ___       _             _     
 *	|   |_ _ _| |___ ___   _| |___ 
 *	| | |_'_| . | -_|  _|_| . | -_|
 *	|___|_,_|___|___|___|_|___|___|
 * 
 * FILE: 	lib_clk.h
 * Author: 	declis (0xdec.de)
 ************************************************/

#ifndef LIB_CLK_H_
#define LIB_CLK_H_

extern unsigned char setup_mode;

// exit setup mode after the specific value
#define setup_end			6

#define str_length			3			// string length for day-name

// LCD positions (x,y)
#define hh_x				8			// hours
#define hh_y				1
#define mm_x				11			// minutes
#define mm_y				1
#define ss_x				14			// seconds
#define ss_y				1
#define day_x				0			// day
#define day_y				0
#define month_x				3			// month
#define month_y				0
#define year_x				6			// year
#define year_y				0
#define day_name_x			0			// day-name
#define day_name_y			1
#define tube_ss_x			5			// seconds tube option
#define tube_ss_y			1
#define c_x					14			// "C"
#define c_y					0
#define temp_c_x			12			// temperature in C
#define temp_c_y			0

// http://www.cosmiq.de/qa/show/1044582/Schaltjahr/?#a3023975 (only for information)
#define cal_start			1900
#define cal_end				2199	


struct the_date
{
	unsigned char day,month,leapyear;
	unsigned int year;
};

struct the_time
{
	// tube_ss = 0 (tube seconds disabled), 1 (tube seconds enabled)
	unsigned char hh,mm,ss,tube_ss;
};

unsigned char get_dayofweek(struct the_date);
unsigned char is_leapyear(unsigned int);
unsigned char reset_day(struct the_date *);
void calc_date(struct the_date *,const char*);
void calc_time(struct the_time *);
void set_lcd_time(struct the_time);
void set_lcd_date(struct the_date,const char[][str_length]);
void set_numitron_time(struct the_time);
void one_tube_off(unsigned char,struct the_time);

#endif /*LIB_CLK_H_*/
