/*************************************************
 *	 ___       _             _     
 *	|   |_ _ _| |___ ___   _| |___ 
 *	| | |_'_| . | -_|  _|_| . | -_|
 *	|___|_,_|___|___|___|_|___|___|
 * 
 * FILE: 	lib_lcd.c
 * Author: 	declis (0xdec.de)
 ************************************************/

#include <msp430g2452.h>
#include "lib_lcd.h"
#include "delay.h"

void clean_area(unsigned char x_pos, unsigned char y_pos, unsigned char number)
{
	// clean the specific area, x,y = start position, number = number of digits
	while(number--)
		write_char(x_pos++,y_pos,' ');
} 

void char_to_num_out(unsigned char x_pos,unsigned char y_pos,unsigned char value)
{
	// only for numbers between 0-99
	write_char(x_pos,y_pos,(value/10)+48);		// write first digit
	write_char(x_pos+1,y_pos,(value%10)+48);	// write second digit 
}

void write_string(unsigned char x_pos,unsigned char y_pos,const char *string)
{
	set_cursor(x_pos,y_pos);					// set cursor to xy-position
	while(*string!=0)							// if string[index]!= '\0' -> write to display
		set_instruction(1,*string++);
}

void write_char(unsigned char x_pos,unsigned char y_pos,unsigned char character)
{
	set_cursor(x_pos,y_pos);					// set cursor to xy-position
	set_instruction(1,character);				// write char
}

void set_cursor(unsigned char x_pos,unsigned char y_pos)
{
	if(y_pos)							
		set_instruction(0,second_row+x_pos);	// cursor @2nd row
	else
		set_instruction(0,first_row+x_pos);		// cursor @1st row
}

void set_instruction(unsigned char register_sel,unsigned char number)
{
	if(register_sel)
		P1OUT|=RS;
	else
		P1OUT&=~RS;
	
	P1OUT&=~CSB;			// start condition
	USISRL=number;			// copy value in low part shift register
	USICNT=8;				// start tranmission (8bit)
}

void init_LCD(void)
{
	wait_ms(100);
	set_instruction(0,0x38);	// 8bit data length,2 lines,instr.table 1
	wait_us(30);
	set_instruction(0,0x39);	// 8bit data length,2 lines,instr.table 1
	wait_us(30);
	set_instruction(0,0x14);	// BS:1/5,2 line LCD
	wait_us(30);
	set_instruction(0,0x55);	// booster on,constrast C5,set C4
	wait_us(30);
	set_instruction(0,0x6D);	// set voltage follower and gain
	wait_ms(250);
	set_instruction(0,0x78);	// set contrast C3,C2,C1
	wait_us(30);
	set_instruction(0,0x38);	// instr.table 0
	wait_us(30);
	set_instruction(0,0x0C);	// display on,cursor off,cursor blink off 0C
	wait_us(30);
	set_instruction(0,0x01);	// delete display, cursor@home
	wait_ms(3);
	set_instruction(0,0x06);	// cursor auto-inc.
	wait_us(30);
}

void init_USI(void)
{	
	P1DIR|=(CSB+RS);			// CSB and RS -> output
	P1OUT|=CSB;					// CSB = high
	P1OUT&=~RS;					// RS = low
	
	// SPI Master Mode (P1.5 CLK, P1.6 SI), Output enabled, reset/stop USI
	USICTL0|=USIPE6+USIPE5+USIMST+USIOE+USISWRST;
	// USI Counter Intterupt enabled
	USICTL1|=USIIE;
	// inactive CLK state -> high, Source = SMCLK = 1MHz
	USICKCTL|=USICKPL+USISSEL_2;
	// start USI
	USICTL0&=~USISWRST;
	// clear USIIFG
	USICTL1&=~USIIFG;
	// global interrupt enabled
	_EINT();
}

#pragma INTERRUPT (USI)
#pragma vector=USI_VECTOR
void USI(void)
{
	USICTL1&=~USIIFG;		// reset USIIFG (transmission done)
	P1OUT|=CSB;				// CSB high state
}	
