/*************************************************
 *	 ___       _             _     
 *	|   |_ _ _| |___ ___   _| |___ 
 *	| | |_'_| . | -_|  _|_| . | -_|
 *	|___|_,_|___|___|___|_|___|___|
 * 
 * FILE: 	lib_system.c
 * Author: 	declis (0xdec.de)
 ************************************************/

#include <msp430g2452.h>
#include "def_ports.h"
#include "lib_system.h"
#include "delay.h"

unsigned char get_temp(void)
{
  	ADC10CTL0|=ENC+ADC10SC;				// start & enable conversion
  	while(ADC10CTL1&ADC10BUSY);			// ADC busy? 

  	// calculate temperature, see 0xdec.de for derivation
  	return(unsigned char)((ADC10MEM*422)/1023)-277;
}

unsigned char set_rgb(unsigned char rgb)
{
	P2OUT&=~(red+green+blue);
	if(rgb>7) rgb=1;
	rgb<<=3;
	P2OUT|=rgb;
	rgb>>=3;	
	return rgb;
}

void send_number(unsigned char number)
{
	number=convert_numitron_bcd(number);
	send_shift_register(number);
}

unsigned char convert_numitron_bcd (unsigned char number)
{
	volatile unsigned char s1=0,s2=0;
	
	s2=number%10;					// split number in two digits (12 -> 1,2)
	s1=number/10;
	number=0;			
	number|=s2+(s1<<=4);			// left-shift 1st digit 4 times (bit 7-4)
									// prepare new byte (two BCD codes)
									// 1st digit: bit 7-4, 2nd digit: bit 3-0
	return number;
}

void send_shift_register(unsigned char byte)
{
	volatile unsigned char bit_num=0;
	
	while(bit_num!=8)				// start transmission
	{
		if(byte&(0x01<<bit_num))	// LSB to MSB
			P2OUT|=data;			// if bit is "1" -> data-port "high"
		else
			P2OUT&=~data;			// if bit is "0" -> data-port "low"
		
		P2OUT|=clock;				// clock pulse
		P2OUT&=~clock;				
		bit_num++;					// increment bit tester
	}
	P2OUT|=strobe;					// transmission done, strobe pulse
	P2OUT&=~strobe;
	P2OUT&=~data;
	bit_num=0;	
}

void init_ADC(void)
{
	// Channel 10 (Temp. Sensor); ADC10 CLK divider /4; CLK=ADC10OSC 
  	ADC10CTL1=INCH_10+ADC10DIV_3+ADC10SSEL_0;     
  	// Vref+=Vr+ and Vss=Vr- ; reference ON (1,5V); S&H time 16x; ADC10 on
  	ADC10CTL0=SREF_1+REFON+ADC10SHT_3+ADC10ON;
}

void init_TimerA(void)
{
	TACTL|=TASSEL_1+ID_0;			// ACLK (32kHz), 1x divider
	TACCR0=cycles_1000;				// interrupt every 1000ms
	TACCTL0|=CCIE;					// enable TimerA interrupt
	TACTL|=MC_1;					// start timer in up-mode
}

void init_ACLK(void)
{
	BCSCTL3|=XCAP_3;				// CL=~12.5pF
	do
	{
		IFG1&=~OFIFG;
		wait_ms(1);
	}
	while(IFG1&OFIFG);
}

void init_DCO(void)
{	
	BCSCTL1=CALBC1_1MHZ;                  
  	DCOCTL=CALDCO_1MHZ;
}

void init_ports(void)
{
	P1DIR&=~(s_mode+s_up+s_down+vcc_check);
	P1IE|=(s_mode+s_up+s_down+vcc_check);
	
	// controller running on battery at start (maybe)
	// a interrupt will be triggered if power supply is available
	// and the LCD will get an init.
	if(P1IN&vcc_check) 
		P1IES|=vcc_check;		// interrupt @ hi/lo edge		
	else P1IES&=~vcc_check;		// interrupt @ lo/hi edge (power supply on)
		
	P1IES&=~(s_mode+s_up+s_down);		
			
	P1IFG&=~(s_mode+s_up+s_down+vcc_check);
	
	P2DIR|=(data+clock+strobe+red+green+blue);
	P2OUT&=~(data+clock+strobe+red+green+blue);
}
